/*
 * \brief  Utility to calculate signatures of TEE messages.
 * \author Pirmin Duss
 * \date   2020-03-03
 */

#pragma once

// libc includes
#include <stdio.h>

// stdcxx includes
#include <string>

// Genode headers
#include <util/string.h>

// openssl / libcrypto headers
#include <openssl/asn1.h>
#include <openssl/conf.h>
#include <openssl/evp.h>
#include <openssl/objects.h>
#include <openssl/x509.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>

using namespace std;

class Libcrypto_wrapper final
{
	private:

		class Ecdsa_private_key
		{
			private:

				string     _key_file_name;
				FILE*      _key_file      { nullptr };
				EVP_PKEY*  _pkey          { nullptr };
				EC_KEY*    _ecdsa         { nullptr };

				Ecdsa_private_key(const Ecdsa_private_key&) = delete;
				Ecdsa_private_key& operator=(const Ecdsa_private_key&) = delete;

				void _close_file();

			public:

				Ecdsa_private_key(const string& file_name);
				virtual ~Ecdsa_private_key();

				EC_KEY* ecdsa() const { return _ecdsa; }
		};

		class Ecdsa_public_key
		{
			private:

				BIO*       _mem    { nullptr };
				X509*      _x509   { nullptr };
				EVP_PKEY*  _pkey   { nullptr };
				EC_KEY*    _ecdsa  { nullptr };

				Ecdsa_public_key(const Ecdsa_public_key&) = delete;
				Ecdsa_public_key& operator=(const Ecdsa_public_key&) = delete;

			public:

				Ecdsa_public_key(const string& certificate_data);
				virtual ~Ecdsa_public_key();

				EC_KEY* ecdsa() const { return _ecdsa; }
		};

	public:

		using Fingerprint_string = Genode::String<128>;

		class File_open_failed { };
		class Invalid_asn1     { };
		class Invalid_key_file { };
		class Invalid_key_type { };
		class Sign_data_failed { };

		Libcrypto_wrapper();
		~Libcrypto_wrapper();

		string sign(const string& data, const string& key_file_name) const;
		string calc_sha256(const string& data) const;

		bool verify(const string& data, const string& signature, const string& certificate_data) const;
		bool verify_asn1(std::string const &data, std::string const &encoded_signature, std::string const &certificate) const;

		Fingerprint_string fingerprint_if_valid(const char* cert_str, size_t size) const;

};
